@@ -361,6 +361,8 @@ pum_display(
 	// redo the positioning.  Limit this to two times, when there is not
 	// much room the window size will keep changing.
     } while (pum_set_selected(selected, redo_count) && ++redo_count <= 2);
+
+    pum_redraw();
 }
 
 /*
@@ -541,8 +543,23 @@ pum_redraw(void)
 			{
 			    if (st != NULL)
 			    {
-				screen_puts_len(st, (int)STRLEN(st), row, col,
-									attr);
+				int size = (int)STRLEN(st);
+				int cells = (*mb_string2cells)(st, size);
+
+				// only draw the text that fits
+				while (size > 0
+					  && col + cells > pum_width + pum_col)
+				{
+				    --size;
+				    if (has_mbyte)
+				    {
+					size -= (*mb_head_off)(st, st + size);
+					cells -= (*mb_ptr2cells)(st + size);
+				    }
+				    else
+					--cells;
+				}
+				screen_puts_len(st, size, row, col, attr);
 				vim_free(st);
 			    }
 			    col += width;
@@ -795,6 +812,10 @@ pum_set_selected(int n, int repeat UNUSED)
 		use_popup = USEPOPUP_NORMAL;
 	    else
 		use_popup = USEPOPUP_NONE;
+	    if (use_popup != USEPOPUP_NONE)
+		// don't use WinEnter or WinLeave autocommands for the info
+		// popup
+		block_autocmds();
 # endif
 	    // Open a preview window and set "curwin" to it.
 	    // 3 lines by default, prefer 'previewheight' if set and smaller.
@@ -972,6 +993,10 @@ pum_set_selected(int n, int repeat UNUSED)
 	    if (WIN_IS_POPUP(curwin))
 		// can't keep focus in a popup window
 		win_enter(firstwin, TRUE);
+# endif
+# ifdef FEAT_PROP_POPUP
+	    if (use_popup != USEPOPUP_NONE)
+		unblock_autocmds();
 # endif
 	}
 #endif
@@ -982,9 +1007,6 @@ pum_set_selected(int n, int repeat UNUSED)
 	popup_hide_info();
 #endif
 
-    if (!resized)
-	pum_redraw();
-
     return resized;
 }
 
